<?php

namespace Realestate\Payment\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Realestate\Payment\Http\Requests\PaymentRequest;
use Realestate\Payment\Interfaces\PaymentRepositoryInterface;
use Realestate\Payment\Models\Payment;

/**
 * Resource controller class for payment.
 */
class PaymentResourceController extends BaseController
{

    /**
     * Initialize payment resource controller.
     *
     * @param type PaymentRepositoryInterface $payment
     *
     * @return null
     */
    public function __construct(PaymentRepositoryInterface $payment)
    {
        parent::__construct();
        $this->repository = $payment;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Realestate\Payment\Repositories\Criteria\PaymentResourceCriteria::class);
    }

    /**
     * Display a list of payment.
     *
     * @return Response
     */
    public function index(PaymentRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Realestate\Payment\Repositories\Presenter\PaymentPresenter::class)
                ->$function();
        }

        $payments = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('payment::payment.names'))
            ->view('payment::payment.index', true)
            ->data(compact('payments', 'view'))
            ->output();
    }

    /**
     * Display payment.
     *
     * @param Request $request
     * @param Model   $payment
     *
     * @return Response
     */
    public function show(PaymentRequest $request, Payment $payment)
    {

        if ($payment->exists) {
            $view = 'payment::payment.show';
        } else {
            $view = 'payment::payment.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('payment::payment.name'))
            ->data(compact('payment'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new payment.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(PaymentRequest $request)
    {

        $payment = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('payment::payment.name')) 
            ->view('payment::payment.create', true) 
            ->data(compact('payment'))
            ->output();
    }

    /**
     * Create new payment.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PaymentRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $payment                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('payment::payment.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('payment/payment/' . $payment->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/payment/payment'))
                ->redirect();
        }

    }

    /**
     * Show payment for editing.
     *
     * @param Request $request
     * @param Model   $payment
     *
     * @return Response
     */
    public function edit(PaymentRequest $request, Payment $payment)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('payment::payment.name'))
            ->view('payment::payment.edit', true)
            ->data(compact('payment'))
            ->output();
    }

    /**
     * Update the payment.
     *
     * @param Request $request
     * @param Model   $payment
     *
     * @return Response
     */
    public function update(PaymentRequest $request, Payment $payment)
    {
        try {
            $attributes = $request->all();

            $payment->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('payment::payment.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('payment/payment/' . $payment->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('payment/payment/' . $payment->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the payment.
     *
     * @param Model   $payment
     *
     * @return Response
     */
    public function destroy(PaymentRequest $request, Payment $payment)
    {
        try {

            $payment->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('payment::payment.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('payment/payment/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('payment/payment/' . $payment->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple payment.
     *
     * @param Model   $payment
     *
     * @return Response
     */
    public function delete(PaymentRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('payment::payment.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('payment/payment'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/payment/payment'))
                ->redirect();
        }

    }

    /**
     * Restore deleted payments.
     *
     * @param Model   $payment
     *
     * @return Response
     */
    public function restore(PaymentRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('payment::payment.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/payment/payment'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/payment/payment/'))
                ->redirect();
        }

    }

}
